/*
* $Author: claudio $
*
* $Revision: 1.1.1.1 $
*
* $Log: multipolynomial.h,v $
* Revision 1.1.1.1  2012-09-05 08:17:22  claudio
* Interpolator utility classes
*
*
*/
#ifndef INTERPOLATOR_MULTIPOLYNOMIAL_H
#define INTERPOLATOR_MULTIPOLYNOMIAL_H

#include <interpolator.h>
#include <interpolatingpolynomial.h>

/**
* @class Interpolator::multiPolynomial
* Class for interpolating fields,currents etc.
* It is used to evalute y=f(x) with x,y belonging to R(1).
* It is implemented by means of a set of interpolating polynomials,
* Each polynomial is valid for the input range [xmini,xmaxi). The polinomial ranges must be contiguos
* and ordered: thath is xmaxi==xmini+1. The range of validity thus becomes [xmin0,xmaxn).
* The ranges are checked by the constructor.
*/



namespace Interpolator{

class multiPolynomial : public Interpolator
{
public:
	
	// Constructors/Destructors
	//
	
	
	/**
	 * Constructor with data
	 * @param polynomials vector of interpolatingPolynomial. must be built outside
	 * @exception std::domain_error: range of polynomial not contiguos or not in order
	 */
	multiPolynomial ( InterpolatingPolynoamialVector& polynomials );
	
	/**
	 * Copy constructor
	 * @param src the multiPolynomial to copied
	 */
	multiPolynomial( multiPolynomial& src);
	
	/**
	 * Assignment operator
	 * @param src the multiPolynomial to copied
	 */
	multiPolynomial& operator=(const multiPolynomial& src);
	
	/**
	 * Destructor
	 */
	~multiPolynomial ( );
	
	/**
	 * Calculate the inetrpolated value y=f(x)
	 * @return interpolated value y
	 * @param  in_val input value (x)
	 * @exception lenght_error wrong number of polynomial (>=1)
	 * @exception range_error at least one input data is outside the [min,max] range
	 */
	double evaluate(double in_val)  throw (std::range_error);
	
	/**
	 * estimate the value of the inverse iterpolator x=g(y)
	 * We make the hypothesys that the interpolated function is invertible!!
	 * @param goal the recquired output (y)
	 * @param initialEstimate initial estimate for x
	 * @return x, NAN if the algorithm did not converge
	 */
	
	double inverse_evaluate(double goal,double initialEstimate);
	
	/**
	 * calculate the interpolated value v[]=f(in_val[]). Vector version
	 * @return doubleVector
	 * @param  in_val doubleVector of input values
	 * @exception range_error: at least one input data is outside the [min,max] range
	 */
	doubleVector evaluate (doubleVector in_val ) throw(std::range_error);
	
protected:
	double* range;       /**< auxiliary struct to search the polynomial to use */
	unsigned int n_poly; /**< number ofinterpolating polynomials polynomials */
	std::vector<doubleVector> _poly; /**< the polynomials, stored as vectors of coefficients */
private:
	
	
};

} //close namespace Interpolator

#endif // INTERPOLATOR_MULTIPOLYNOMIAL_H
