/*
 * class multiPolynomial - implementation
 *
 * $Author: claudio $
 *
 * $Revision: 1.2 $
 *
 * $Log: multipolynomial.cpp,v $
 * Revision 1.2  2017-03-03 15:32:26  claudio
 * correctetd   spline::inverse_calculate if returned value is at range limit
 *
 * Revision 1.1.1.1  2012-09-05 08:17:22  claudio
 * Interpolator utility classes
 *
 */

#include "multipolynomial.h"
#include <stdexcept>

namespace Interpolator{

multiPolynomial::multiPolynomial ( InterpolatingPolynoamialVector& poly){
	n_poly=poly.size();
	if( ! n_poly ) throw std::length_error("multiPolynomial(): at least one polynomial is recquired");
	
	//scan limits and check that they are in order and continguos
	{
		double low,high;
		unsigned int i=0;
		low=poly[i].xMin;
		high=poly[i].xMax;
		while(++i < n_poly){
			if(low >= high) throw std::domain_error("multiPolynomial: xMin >= xMax");
			if(high != poly[i].xMin) throw std::domain_error("multiPolynomial: not contiguos");
			low=poly[i].xMin;
			high=poly[i].xMax;
		}
	}
	//good - create structure to store the data
	range = new double[n_poly+1];
	range[0]=poly[0].xMin; //first is special
	for (unsigned int i=0; i<n_poly ; i++){
		range[i+1]=poly[i].xMax;
		_poly.push_back(poly[i].coefficient);
	} 
	Interpolator::_min=range[0];
	Interpolator::_max=range[n_poly];
}


multiPolynomial::multiPolynomial(multiPolynomial& src):Interpolator()
{
	_min=src._min;
	_max=src._max;
	n_poly=src.n_poly;
	range = new double[n_poly+1];
	range[0]=src.range[0];
	for (unsigned int i=0; i<n_poly ; i++){
		range[i+1]=src.range[i+1];
	}
	_poly=src._poly; //exploit assignment operator from stl
}

multiPolynomial& multiPolynomial::operator=(const multiPolynomial& src)
{
	_min=src._min;
	_max=src._max;
	n_poly=src.n_poly;
	range = new double[n_poly+1];
	range[0]=src.range[0];
	for (unsigned int i=0; i<n_poly ; i++){
		range[i+1]=src.range[i+1];
	}
	_poly=src._poly; //exploit assignment operator from stl
	return *this;
}

multiPolynomial::~multiPolynomial ( )
{
	delete [] range;
}

double multiPolynomial::evaluate(double v)  throw (std::range_error)
{
	if ( v < _min  || v > _max ) throw std::range_error("input value out of range");
	//find the correct polynomial to use

	unsigned int klo,khi,k;
	klo=0;
	khi=1;
	while( khi < n_poly  ){
		if( v< range[khi] && v >= range[klo]) break;
		klo++;khi++;
	}
	k=klo;

	// Horner rule - can also evaluate the derivative
	unsigned int degree=_poly[k].size()-1;
	doubleVector* poly=&_poly[k];
	double p=poly->at(degree);;
	//double dp=0; - derivative , disabled
	for( int i= degree-1; i >= 0; i-- ){
		p  = v*p + poly->at(i);
		//dp = v*dp + p; //derivative evaluation
	}
	return p;
}

doubleVector multiPolynomial::evaluate (doubleVector in_val ) throw(std::range_error)
{
	//simplicistic implementation. Should make better use of the indexing
	doubleVector outval;
	for(unsigned int i=0; i <  in_val.size(); i++) outval.push_back(evaluate(in_val[i]));
	return outval;
}

double multiPolynomial::inverse_evaluate(double goal,double initialEstimate)
{
	//dirty work-around untill the algorithm is tested also for multy-polinomial!!!
	//TODO : implement inverse_evaluate
	return initialEstimate;
}

} // close namespace Interpolator
